"use strict";

exports.__esModule = true;
exports.TransformImportsToCommonRoot = undefined;

var _path = require("path");

var resolve = require("resolve");

var requireToPath = function requireToPath(req, filedir, basedir) {
  return (0, _path.relative)(basedir, resolve.sync(req, {
    basedir: filedir
  }));
};

var TransformImportsToCommonRoot = exports.TransformImportsToCommonRoot = function TransformImportsToCommonRoot(module) {
  if (module === void 0) {
    module = {};
  }

  return function () {
    function isValidRequireCall(nodepath) {
      if (!nodepath.isCallExpression()) return false;

      if (!nodepath.get("callee").isIdentifier({
        name: "require"
      })) {
        return false;
      }

      if (nodepath.scope.getBinding("require")) return false;
      var args = nodepath.get("arguments");
      if (args.length !== 1) return false;
      var arg = args[0];
      if (!arg.isStringLiteral()) return false;
      return true;
    }

    var AmdVisitor = function AmdVisitor(filedir, basedir, dependencyPaths) {
      return {
        CallExpression: function CallExpression(nodepath) {
          if (!isValidRequireCall(nodepath)) return;
          var req = nodepath.node.arguments[0].value;
          var newPath = requireToPath(req, filedir, basedir);
          nodepath.node.arguments[0].value = newPath;
          dependencyPaths.push(newPath);
        }
      };
    };

    return {
      visitor: {
        Program: {
          exit: function exit(nodepath, _ref) {
            var cwd = _ref.cwd,
                filename = _ref.filename;
            if (this.ran) return;
            this.ran = true;
            var filedir = (0, _path.dirname)(filename);
            module.dependencyPaths = [];
            nodepath.traverse(AmdVisitor(filedir, cwd, module.dependencyPaths), this);
          }
        }
      }
    };
  };
};

exports.default = TransformImportsToCommonRoot();